@@ -1,5 +1,6 @@
 glmm <-
-function(fixed,random, varcomps.names,data, family.glmm, m,varcomps.equal, doPQL=TRUE, debug=FALSE,p1=1/3,p2=1/3,p3=1/3,rmax=1000,iterlim=1000,par.init=NULL,zeta=5){
+function(fixed,random, varcomps.names,data, family.glmm, m,varcomps.equal, doPQL=TRUE, debug=FALSE,p1=1/3,p2=1/3,p3=1/3,rmax=1000,iterlim=1000,par.init=NULL,zeta=5)
+	{
 	if(missing(varcomps.names)) stop("Names for the variance components must be supplied through varcomps.names")
 	if(is.vector(varcomps.names)!=1) stop("varcomps.names must be a vector")
 	if(missing(varcomps.equal)){
@@ -17,6 +18,37 @@ function(fixed,random, varcomps.names,data, family.glmm, m,varcomps.equal, doPQL
     }
     x <- model.matrix(fixed, data = barf)
     y <- model.response(barf)
+
+
+	#family stuff
+	family.glmm<-getFamily(family.glmm)
+
+	#check that only binomial has y being matrix
+	if(length(dim(y))==2){
+		if(family.glmm$family.glmm!="binomial.glmm") {
+			stop("For the family you've specified, only a vector is appropriate as the response. Binomial is the only family that allows you to specify the response as a matrix.")
+		}	
+	}
+
+	ntrials <- rep(1, length(y)) #used for Poisson and Bern, essentially untouched
+	if(family.glmm$family.glmm=="binomial.glmm"){
+		#if the response is a vector, then ntrials stays at 1
+		#if the response is a matrix
+		if(length(dim(y))==2){
+			#make sure it has exactly 2 columns
+			if(ncol(y)!=2) stop("Your response must have two columns: the first column reports the number of successes and the second column reports the number of failures.")
+
+			# make ntrials a vector with each entry the sum of the entries in the corresponding col of y
+			ntrials <- apply(y, MARGIN=1, FUN=sum)
+			y <- y[,1] 	#then change y to just be the number of successes
+		}
+
+
+	}
+	#do the check for the specified family
+	family.glmm$checkData(y)
+
+
     
     #then the part for the random effects. 
     #first, if it's not a list, make it a list 
@@ -25,23 +57,56 @@ function(fixed,random, varcomps.names,data, family.glmm, m,varcomps.equal, doPQL
         random <- list(random)
     #put this stuff in a loop and loop along the list
     #for i in 1:length(formula2)
-    for (irandom in seq(along = random)) {
+    for (irandom in seq(along = random)) 
+			{
     		r<-random[[irandom]]
     		stopifnot(inherits(r, "formula"))
     		if (missing(data)) {
-       		 barf2 <- lm(r, method = "model.frame")
-   		} else {
-       		 stopifnot(inherits(data, "data.frame"))
-        	barf2 <- lm(r, data = data, method = "model.frame")
+       			barf2 <- lm(r, method = "model.frame")
+   			} else {
+       			stopifnot(inherits(data, "data.frame"))
+        		barf2 <- lm(r, data = data, method = "model.frame")
     		}
     	random[[irandom]] <- model.matrix(r, data = barf2)
 	#thisgroup<-varcomps.equal[irandom]
 	#names(random)[irandom]<-varcomps.names[thisgroup]
 
-	if(length(y)!=nrow(random[[irandom]])) stop("Fixed and random effect model matrices should have same number of rows")
+	if(length(y)!=nrow(random[[irandom]])) {
+		stop("Fixed and random effect model matrices should have same number of rows. This problem sometimes arises due to NAs (missing data).")
+		}
 	}
 	#so now random is a list containing a model matrix for each formula, and some matrices share variance components
 
+#	#family stuff
+#	family.glmm<-getFamily(family.glmm)
+
+#	#check that only binomial has y being matrix
+#	if(length(dim(y))==2){
+#		if(family.glmm$family.glmm!="binomial.glmm") {
+#			stop("For the family you've specified, only a vector is appropriate as the response. Binomial is the only family that allows you to specify the response as a matrix.")
+#		}	
+#	}
+
+#	ntrials <- rep(1, length(y)) #used for Poisson and Bern, essentially untouched
+#	if(family.glmm$family.glmm=="binomial.glmm"){
+#		#if the response is a vector, then ntrials stays at 1
+#		#if the response is a matrix
+#		if(length(dim(y))==2){
+#			#make sure it has exactly 2 columns
+#			if(ncol(y)!=2) stop("Your response must have two columns: the first column reports the number of successes and the second column reports the number of failures.")
+
+#			# make ntrials a vector with each entry the sum of the entries in the corresponding col of y
+#			ntrials <- apply(y, MARGIN=1, FUN=sum)
+#			y <- y[,1] 	#then change y to just be the number of successes
+#		}
+
+
+#	}
+#	#do the check for the specified family
+#	family.glmm$checkData(y)
+
+
+
 	if(is.numeric(varcomps.equal)==F) stop("varcomps.equal must be a vector containing numbers to indicate which variance components are equal.")
 	if(length(varcomps.equal)!=length(random)){
 		stop("The length of varcomps.equal must be equal to the length of the random-effects call.")} 
@@ -49,8 +114,9 @@ function(fixed,random, varcomps.names,data, family.glmm, m,varcomps.equal, doPQL
 		stop("You must name each unique variance component. Check varcomps.names and varcomps.equal.")} 
 	if(min(varcomps.equal)!=1)stop("The vector varcomps.equal must contain numbers starting at 1 to denote which variance components are equal.")	
 	levs<-ordered(unique(varcomps.equal))
-	family.glmm<-getFamily(family.glmm)
-	family.glmm$checkData(y)
+
+
+
 
 
 	#check p1 p2 p3
@@ -78,7 +144,7 @@ function(fixed,random, varcomps.names,data, family.glmm, m,varcomps.equal, doPQL
 	}
 	names(z)<-varcomps.names
 
-	mod.mcml<-list(x = x, z=z,y = y)
+	mod.mcml<-list(x = x, z=z, y = y, ntrials = ntrials)
 
 	
 	#so now the 3 items are x (matrix), z (list), y (vector)
@@ -180,7 +246,10 @@ function(fixed,random, varcomps.names,data, family.glmm, m,varcomps.equal, doPQL
 	if(m3>0){
 		Z=do.call(cbind,mod.mcml$z)
 		eta.star<-as.vector(mod.mcml$x%*%beta.pql+Z%*%u.star)
-		cdouble<-family.glmm$cpp(eta.star) #still a vector
+		if(family.glmm$family.glmm=="bernoulli.glmm") {cdouble<-family.glmm$cpp(eta.star)}
+		if(family.glmm$family.glmm=="poisson.glmm"){cdouble<-family.glmm$cpp(eta.star)}
+		if(family.glmm$family.glmm=="binomial.glmm"){cdouble<-family.glmm$cpp(eta.star, ntrials)}
+		 #still a vector
 		cdouble<-Diagonal(length(cdouble),cdouble)
 		Sigmuh.inv<- t(Z)%*%cdouble%*%Z+D.star.inv
 		Sigmuh<-solve(Sigmuh.inv)
@@ -199,8 +268,7 @@ function(fixed,random, varcomps.names,data, family.glmm, m,varcomps.equal, doPQL
 	umat<-rbind(genData,genData2,genData3)
 
 	#use trust to max the objfun (monte carlo likelihood)
-	trust.out<-trust(objfun,parinit=par.init,rinit=10, minimize=FALSE, rmax=rmax, iterlim=iterlim, blather=debug, nbeta=length(beta.pql), nu.pql=nu.pql, 
-umat=umat, mod.mcml=mod.mcml, family.glmm=family.glmm, u.star=u.star,  cache=cache,  p1=p1,p2=p2, p3=p3,m1=m1, D.star=D.star,Sigmuh=Sigmuh,Sigmuh.inv=Sigmuh.inv,zeta=zeta)
+	trust.out<-trust(objfun,parinit=par.init,rinit=10, minimize=FALSE, rmax=rmax, iterlim=iterlim, blather=debug, nbeta=length(beta.pql), nu.pql=nu.pql, umat=umat, mod.mcml=mod.mcml, family.glmm=family.glmm, u.star=u.star,  cache=cache,  p1=p1,p2=p2, p3=p3,m1=m1, D.star=D.star, Sigmuh=Sigmuh, Sigmuh.inv=Sigmuh.inv, zeta=zeta, ntrials = ntrials)
 
 	beta.trust<-trust.out$argument[1:length(beta.pql)]
 	nu.trust<-trust.out$argument[-(1:length(beta.pql))]
@@ -216,9 +284,11 @@ umat=umat, mod.mcml=mod.mcml, family.glmm=family.glmm, u.star=u.star,  cache=cac
 	debug<-list(beta.pql=beta.pql, nu.pql=nu.pql,  trust.argpath=trust.out$argpath, u.star=u.star, umat=umat,weights=cache$weights,wtsnumer=cache$numer,wtsdenom=cache$denom,m1=m1,m2=m2,m3=m3,trust.argtry=trust.out$argtry, trust.steptype=trust.out$steptype, trust.accept=trust.out$accept, trust.r=trust.out$r, trust.rho=trust.out$rho, trust.valpath=trust.out$valpath, trust.valtry=trust.out$valtry, trust.preddif=trust.out$preddif, trust.stepnorm=trust.out$stepnorm)
 	}
 	
+
+
 	return(structure(list(beta=beta.trust,nu=nu.trust, likelihood.value=trust.out$value, likelihood.gradient=trust.out$gradient, likelihood.hessian=trust.out$hessian,
 	trust.converged=trust.out$converged,  mod.mcml=mod.mcml,
 	 fixedcall=fixed,randcall=randcall, x=x,y=y, z=random,
 	family.glmm=family.glmm, call=call, varcomps.names=varcomps.names, 
-	varcomps.equal=varcomps.equal, debug=debug), class="glmm"))
+	varcomps.equal=varcomps.equal, umat=umat, pvec=c(p1, p2, p3), beta.pql=beta.pql, nu.pql=nu.pql, u.pql=u.star, zeta=zeta, debug=debug), class="glmm"))
 }
